from gtcadis_tools import calc_T
from pyne.mesh import Mesh, IMeshTag
from pyne.dagmc import load, discretize_geom, cell_material_assignments
from pyne.material import MaterialLibrary
import numpy as np

geom = "../../geom/neutron/neutron.h5m"
adj_p = "../adj_photon/adj_p.h5m"
mesh = Mesh(structured=True, mesh=adj_p)

load(geom)
cell_mats_raw = cell_material_assignments(geom)
cell_mats = {c:int(m.split(":")[1].split("/")[0][1]) if c in [2, 3, 4] else 0 for c, m in cell_mats_raw.items()}

neutron_spectrum = [1]*175 # will be normalized
flux_magnitudes = [1.75E14] # 1E12*175
# single pulse irradiation scenario of interest
decay_times = [1E5]
irr_times = [1E5]

# T(mat, decay_time, n, p)
ml = MaterialLibrary(geom)
mats = [ml['mat:m1/rho:7.930000E+00'], ml['mat:m2/rho:1.205000E-03']]

T = calc_T(mats, neutron_spectrum, irr_times, flux_magnitudes, decay_times, remove=True)
np.set_printoptions(threshold=np.nan)

mesh.flux = IMeshTag(217)
mesh.adj_n_src = IMeshTag(217)

adj_p = mesh.flux[:]

temp = np.zeros(shape=(len(mesh), 217))

dg = discretize_geom(mesh)
for i in range(len(mesh)):
    for row in np.atleast_1d(dg[dg["idx"] == i]):
        cell = row[1]
        mat = cell_mats[cell] - 1
        for g in range(175):
            for h in range(42):
                temp[i, g + 42] += adj_p[i, h]*T[mat, 0, g, h]
mesh.adj_n_src[:] = temp
mesh.mesh.save("adj_n_src.h5m")
